"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Agile = void 0;
const internal_1 = require("./internal");
class Agile {
    /**
     * @public
     * Agile - Global state and logic framework for reactive Typescript & Javascript applications
     * @param config - Config
     */
    constructor(config = {}) {
        //=========================================================================================================
        // Storage
        //=========================================================================================================
        /**
         * @public
         * Storage - Handy Interface for storing Items permanently
         * @param config - Config
         */
        this.Storage = (config) => new internal_1.Storage(config);
        //=========================================================================================================
        // State
        //=========================================================================================================
        /**
         * @public
         * State - Class that holds one Value and causes rerender on subscribed Components
         * @param initialValue - Initial Value of the State
         * @param config - Config
         */
        this.State = (initialValue, config = {}) => new internal_1.State(this, initialValue, config);
        //=========================================================================================================
        // Collection
        //=========================================================================================================
        /**
         * @public
         * Collection - Class that holds a List of Objects with key and causes rerender on subscribed Components
         * @param config - Config
         */
        this.Collection = (config) => new internal_1.Collection(this, config);
        //=========================================================================================================
        // Computed
        //=========================================================================================================
        /**
         * @public
         * Computed - Function that recomputes its value if a dependency changes
         * @param computeFunction - Function for computing value
         * @param deps - Hard coded dependencies of Computed Function
         */
        this.Computed = (computeFunction, deps) => new internal_1.Computed(this, computeFunction, {
            computedDeps: deps,
        });
        //=========================================================================================================
        // Event
        //=========================================================================================================
        /**
         * @public
         * Event - Class that holds a List of Functions which can be triggered at the same time
         * @param config - Config
         */
        this.Event = (config) => new internal_1.Event(this, config);
        config = internal_1.defineConfig(config, {
            localStorage: true,
            waitForMount: false,
            logConfig: {},
        });
        config.logConfig = internal_1.defineConfig(config.logConfig, {
            prefix: 'Agile',
            active: true,
            level: internal_1.Logger.level.WARN,
            canUseCustomStyles: true,
            allowedTags: ['runtime', 'storage', 'subscription', 'multieditor'],
        });
        this.config = {
            waitForMount: config.waitForMount,
        };
        this.integrations = new internal_1.Integrations(this);
        this.runtime = new internal_1.Runtime(this);
        this.subController = new internal_1.SubController(this);
        this.storages = new internal_1.Storages(this, {
            localStorage: config.localStorage,
        });
        // Assign customized config to Logger
        Agile.logger = new internal_1.Logger(config.logConfig);
        // Create global instance of Agile
        if (!internal_1.globalBind('__agile__', this))
            Agile.logger.warn('Be careful with multiple Agile Instances in one Application!');
    }
    //=========================================================================================================
    // Integrate
    //=========================================================================================================
    /**
     * @public
     * Integrates framework into Agile
     * @param integration - Integration that gets registered/integrated
     */
    integrate(integration) {
        this.integrations.integrate(integration);
        return this;
    }
    //=========================================================================================================
    // Register Storage
    //=========================================================================================================
    /**
     * @public
     * Registers new Storage as Agile Storage
     * @param storage - new Storage
     * @param config - Config
     */
    registerStorage(storage, config = {}) {
        this.storages.register(storage, config);
        return this;
    }
    //=========================================================================================================
    // Has Integration
    //=========================================================================================================
    /**
     * @public
     * Checks if Agile has any registered Integration
     */
    hasIntegration() {
        return this.integrations.hasIntegration();
    }
    //=========================================================================================================
    // Has Storage
    //=========================================================================================================
    /**
     * @public
     * Checks if Agile has any registered Storage
     */
    hasStorage() {
        return this.storages.hasStorage();
    }
}
exports.Agile = Agile;
Agile.initialIntegrations = []; // External added initial Integrations
// Static Logger with default config -> will be overwritten by config of last created Agile Instance
Agile.logger = new internal_1.Logger({
    prefix: 'Agile',
    active: true,
    level: internal_1.Logger.level.WARN,
});
