"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionPersistent = void 0;
const internal_1 = require("../internal");
class CollectionPersistent extends internal_1.Persistent {
    /**
     * @internal
     * Collection Persist Manager - Handles permanent storing of Collection Value
     * @param collection - Collection that gets stored
     * @param config - Config
     */
    constructor(collection, config = {}) {
        super(collection.agileInstance(), {
            instantiate: false,
        });
        config = internal_1.defineConfig(config, {
            instantiate: true,
            storageKeys: [],
        });
        this.collection = () => collection;
        this.instantiatePersistent({
            key: config.key,
            storageKeys: config.storageKeys,
        });
        // Load/Store persisted Value/s for the first Time
        if (this.ready && config.instantiate)
            this.initialLoading();
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @internal
     * Updates Key/Name of Persistent
     * @param value - New Key/Name of Persistent
     */
    setKey(value) {
        return __awaiter(this, void 0, void 0, function* () {
            const oldKey = this._key;
            const wasReady = this.ready;
            // Assign Key
            if (value === this._key)
                return;
            this._key = value || internal_1.Persistent.placeHolderKey;
            const isValid = this.validatePersistent();
            // Try to Initial Load Value if persistent wasn't ready
            if (!wasReady) {
                if (isValid)
                    yield this.initialLoading();
                return;
            }
            // Remove value at old Key
            yield this.removePersistedValue(oldKey);
            // Assign Value to new Key
            if (isValid)
                yield this.persistValue(value);
        });
    }
    //=========================================================================================================
    // Initial Loading
    //=========================================================================================================
    /**
     * @internal
     * Loads/Saves Storage Value for the first Time
     */
    initialLoading() {
        const _super = Object.create(null, {
            initialLoading: { get: () => super.initialLoading }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.initialLoading.call(this).then(() => {
                this.collection().isPersisted = true;
            });
        });
    }
    //=========================================================================================================
    // Load Persisted Value
    //=========================================================================================================
    /**
     * @internal
     * Loads Collection from Storage
     * @param key - Prefix Key of Persisted Instances (default PersistentKey)
     * @return Success?
     */
    loadPersistedValue(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _key = key || this._key;
            // Check if Collection is Persisted
            const isPersisted = yield this.agileInstance().storages.get(_key, this.defaultStorageKey);
            if (!isPersisted)
                return false;
            // Loads Values into Collection
            const loadValuesIntoCollection = () => __awaiter(this, void 0, void 0, function* () {
                var _a, _b;
                const defaultGroup = this.collection().getGroup(this.collection().config.defaultGroupKey);
                if (!defaultGroup)
                    return false;
                // Persist Default Group and load its Value manually to be 100% sure it got loaded
                defaultGroup.persist({
                    instantiate: false,
                    followCollectionPersistKeyPattern: true,
                });
                if ((_a = defaultGroup.persistent) === null || _a === void 0 ? void 0 : _a.ready) {
                    yield ((_b = defaultGroup.persistent) === null || _b === void 0 ? void 0 : _b.initialLoading());
                    defaultGroup.isPersisted = true;
                }
                // Load Items into Collection
                for (const itemKey of defaultGroup._value) {
                    const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, _key);
                    // Get Storage Value
                    const storageValue = yield this.agileInstance().storages.get(itemStorageKey, this.defaultStorageKey);
                    if (!storageValue)
                        continue;
                    // Collect found Storage Value
                    this.collection().collect(storageValue);
                }
                return true;
            });
            const success = yield loadValuesIntoCollection();
            // Persist Collection, so that the Storage Value updates dynamically if the Collection updates
            if (success)
                yield this.persistValue(_key);
            return success;
        });
    }
    //=========================================================================================================
    // Persist Value
    //=========================================================================================================
    /**
     * @internal
     * Sets everything up so that the Collection gets saved in the Storage
     * @param key - Prefix Key of Persisted Instances (default PersistentKey)
     * @return Success?
     */
    persistValue(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _key = key || this._key;
            const defaultGroup = this.collection().getGroup(this.collection().config.defaultGroupKey);
            if (!defaultGroup)
                return false;
            // Set Collection to Persisted (in Storage)
            this.agileInstance().storages.set(_key, true, this.storageKeys);
            // Persist default Group
            if (!defaultGroup.isPersisted)
                defaultGroup.persist({ followCollectionPersistKeyPattern: true });
            // Add sideEffect to default Group which adds and removes Items from the Storage depending on the Group Value
            defaultGroup.addSideEffect(CollectionPersistent.defaultGroupSideEffectKey, () => this.rebuildStorageSideEffect(defaultGroup, _key));
            // Persist Collection Items
            for (const itemKey of defaultGroup._value) {
                const item = this.collection().getItem(itemKey);
                const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, _key);
                item === null || item === void 0 ? void 0 : item.persist(itemStorageKey);
            }
            this.isPersisted = true;
            return true;
        });
    }
    //=========================================================================================================
    // Remove Persisted Value
    //=========================================================================================================
    /**
     * @internal
     * Removes Collection from the Storage
     * @param key - Prefix Key of Persisted Instances (default PersistentKey)
     * @return Success?
     */
    removePersistedValue(key) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _key = key || this._key;
            const defaultGroup = this.collection().getGroup(this.collection().config.defaultGroupKey);
            if (!defaultGroup)
                return false;
            // Set Collection to not Persisted
            this.agileInstance().storages.remove(_key, this.storageKeys);
            // Remove default Group from Storage
            (_a = defaultGroup.persistent) === null || _a === void 0 ? void 0 : _a.removePersistedValue();
            // Remove Rebuild Storage sideEffect from default Group
            defaultGroup.removeSideEffect(CollectionPersistent.defaultGroupSideEffectKey);
            // Remove Collection Items from Storage
            for (const itemKey of defaultGroup._value) {
                const item = this.collection().getItem(itemKey);
                (_b = item === null || item === void 0 ? void 0 : item.persistent) === null || _b === void 0 ? void 0 : _b.removePersistedValue();
            }
            this.isPersisted = false;
            return true;
        });
    }
    //=========================================================================================================
    // Format Key
    //=========================================================================================================
    /**
     * @internal
     * Formats Storage Key
     * @param key - Key that gets formatted
     */
    formatKey(key) {
        const collection = this.collection();
        // Get key from Collection
        if (!key && collection._key)
            return collection._key;
        if (!key)
            return;
        // Set Storage Key to Collection Key if Collection has no key
        if (!collection._key)
            collection._key = key;
        return key;
    }
    //=========================================================================================================
    // Rebuild Storage SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Rebuilds Storage depending on Group
     * @param group - Group
     * @param key - Prefix Key of Persisted Instances (default PersistentKey)
     */
    rebuildStorageSideEffect(group, key) {
        var _a;
        const collection = group.collection();
        const _key = key || ((_a = collection.persistent) === null || _a === void 0 ? void 0 : _a._key);
        // Return if only a ItemKey got updated
        if (group.previousStateValue.length === group._value.length)
            return;
        const addedKeys = group._value.filter((key) => !group.previousStateValue.includes(key));
        const removedKeys = group.previousStateValue.filter((key) => !group._value.includes(key));
        // Persist Added Keys
        addedKeys.forEach((itemKey) => {
            var _a;
            const item = collection.getItem(itemKey);
            const _itemKey = CollectionPersistent.getItemStorageKey(itemKey, _key);
            if (!item)
                return;
            if (!item.isPersisted)
                item.persist(_itemKey);
            else
                (_a = item.persistent) === null || _a === void 0 ? void 0 : _a.persistValue(_itemKey);
        });
        // Unpersist removed Keys
        removedKeys.forEach((itemKey) => {
            var _a;
            const item = collection.getItem(itemKey);
            const _itemKey = CollectionPersistent.getItemStorageKey(itemKey, _key);
            if (!item)
                return;
            if (item.isPersisted)
                (_a = item.persistent) === null || _a === void 0 ? void 0 : _a.removePersistedValue(_itemKey);
        });
    }
    //=========================================================================================================
    // Get Item Storage Key
    //=========================================================================================================
    /**
     * @internal
     * Build Item StorageKey with Collection Persist Pattern
     * @param itemKey - Key of Item
     * @param collectionKey - Key of Collection
     */
    static getItemStorageKey(itemKey, collectionKey) {
        if (!itemKey || !collectionKey)
            internal_1.Agile.logger.warn('Failed to build unique Item StorageKey!');
        if (!itemKey)
            itemKey = 'unknown';
        if (!collectionKey)
            collectionKey = 'unknown';
        return this.storageItemKeyPattern
            .replace('${collectionKey}', collectionKey.toString())
            .replace('${itemKey}', itemKey.toString());
    }
    //=========================================================================================================
    // Get Group Storage Key
    //=========================================================================================================
    /**
     * @internal
     * Build Group StorageKey with Collection Persist Pattern
     * @param groupKey - Key of Group
     * @param collectionKey - Key of Collection
     */
    static getGroupStorageKey(groupKey, collectionKey) {
        if (!groupKey || !collectionKey)
            internal_1.Agile.logger.warn('Failed to build unique Group StorageKey!');
        if (!groupKey)
            groupKey = 'unknown';
        if (!collectionKey)
            collectionKey = 'unknown';
        return this.storageGroupKeyPattern
            .replace('${collectionKey}', collectionKey.toString())
            .replace('${groupKey}', groupKey.toString());
    }
}
exports.CollectionPersistent = CollectionPersistent;
CollectionPersistent.defaultGroupSideEffectKey = 'rebuildGroupStorageValue';
CollectionPersistent.storageItemKeyPattern = '_${collectionKey}_item_${itemKey}';
CollectionPersistent.storageGroupKeyPattern = '_${collectionKey}_group_${groupKey}';
