"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Selector = void 0;
const internal_1 = require("../internal");
class Selector extends internal_1.State {
    /**
     * @public
     * Represents Item of Collection
     * @param collection - Collection that contains the Item
     * @param itemKey - ItemKey of Item that the Selector represents
     * @param config - Config
     */
    constructor(collection, itemKey, config = {}) {
        super(collection.agileInstance(), undefined, config);
        config = internal_1.defineConfig(config, {
            isPlaceholder: false,
        });
        this.collection = () => collection;
        this.item = undefined;
        this._itemKey = Selector.dummyItemKey;
        this._key = config === null || config === void 0 ? void 0 : config.key;
        this.isPlaceholder = true;
        // Initial Select
        if (!config.isPlaceholder)
            this.select(itemKey, { overwrite: true });
    }
    /**
     * @public
     * Set ItemKey that the Selector represents
     */
    set itemKey(value) {
        this.select(value);
    }
    /**
     * @public
     * Get ItemKey that the Selector represents
     */
    get itemKey() {
        return this._itemKey;
    }
    //=========================================================================================================
    // Select
    //=========================================================================================================
    /**
     * @public
     * Select new ItemKey
     * @param itemKey - New ItemKey
     * @param config - Config
     */
    select(itemKey, config = {}) {
        const oldItem = this.collection().getItem(this._itemKey, {
            notExisting: true,
        }); // Because this.item might be outdated
        const newItem = this.collection().getItemWithReference(itemKey);
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: true,
            force: false,
            overwrite: (oldItem === null || oldItem === void 0 ? void 0 : oldItem.isPlaceholder) || false,
            storage: true,
        });
        if (this.hasSelected(itemKey) && !config.force) {
            internal_1.Agile.logger.warn(`Selector has already selected '${itemKey}'!`);
            return this;
        }
        // Unselect old Item
        this.unselect({ background: true });
        this._itemKey = itemKey;
        this.item = newItem;
        newItem.isSelected = true;
        // Add SideEffect to newItem, that rebuild this Selector depending on the current Item Value
        newItem.addSideEffect(Selector.rebuildSelectorSideEffectKey, (config) => this.rebuildSelector(config));
        // Rebuild Selector for instantiating new 'selected' ItemKey properly
        this.rebuildSelector(config);
        return this;
    }
    //=========================================================================================================
    // Unselect
    //=========================================================================================================
    /**
     * @public
     * Unselects current selected Item
     * @param config - Config
     */
    unselect(config = {}) {
        // Because this.item might be outdated
        const item = this.collection().getItem(this._itemKey, {
            notExisting: true,
        });
        // Unselect Item
        if (item) {
            item.isSelected = false;
            item.removeSideEffect(Selector.rebuildSelectorSideEffectKey);
            if (item.isPlaceholder)
                delete this.collection().data[this._itemKey];
        }
        // Reset and rebuild Selector
        this.item = undefined;
        this._itemKey = Selector.dummyItemKey;
        this.rebuildSelector(config);
        this.isPlaceholder = true;
        return this;
    }
    //=========================================================================================================
    // Has Selected
    //=========================================================================================================
    /**
     * Checks if Selector has selected passed ItemKey
     * @param itemKey
     */
    hasSelected(itemKey) {
        const isSelected = this._itemKey === itemKey;
        if (!this.item)
            return isSelected;
        return isSelected && this.item.isSelected;
    }
    //=========================================================================================================
    // Rebuild Selector
    //=========================================================================================================
    /**
     * @public
     * Rebuilds Selector
     * @param config - Config
     */
    rebuildSelector(config = {}) {
        // Set Selector Value to undefined if Item doesn't exist
        if (!this.item || this.item.isPlaceholder) {
            this.set(undefined, config);
            return;
        }
        // Set Selector Value to updated Item Value
        this.set(this.item._value, config);
    }
}
exports.Selector = Selector;
Selector.dummyItemKey = 'unknown';
Selector.rebuildSelectorSideEffectKey = 'rebuildSelector';
