"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Event = void 0;
const internal_1 = require("../internal");
const event_observer_1 = require("./event.observer");
class Event {
    /**
     * @public
     * Event -  Class that holds a List of Functions which can be triggered at the same time
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance, config = {}) {
        this.uses = 0;
        this.callbacks = {}; // All 'subscribed' callback function
        this.enabled = true;
        this.queue = []; // Queue of delayed Events
        this.agileInstance = () => agileInstance;
        config = internal_1.defineConfig(config, {
            enabled: true,
            rerender: false,
            maxUses: undefined,
            delay: undefined,
            overlap: false,
            deps: [],
        });
        this._key = config.key;
        this.observer = new event_observer_1.EventObserver(this, {
            key: config.key,
            deps: config.deps,
        });
        this.enabled = config.enabled;
        this.config = {
            rerender: config.rerender,
            delay: config.delay,
            maxUses: config.maxUses,
            overlap: config.overlap,
        };
        this.initialConfig = config;
    }
    /**
     * @public
     * Set Key/Name of Event
     */
    set key(value) {
        this.setKey(value);
    }
    /**
     * @public
     * Get Key/Name of Event
     */
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @internal
     * Set Key/Name of Event
     * @param value - New Key/Name of Event
     */
    setKey(value) {
        this._key = value;
        this.observer._key = value;
        return this;
    }
    on(keyOrCallback, callback) {
        const generateKey = internal_1.isFunction(keyOrCallback);
        let _callback;
        let key;
        if (generateKey) {
            key = internal_1.generateId();
            _callback = keyOrCallback;
        }
        else {
            key = keyOrCallback;
            _callback = callback;
        }
        // Check if Callback is a Function
        if (!internal_1.isFunction(_callback)) {
            internal_1.Agile.logger.error('A Event Callback Function has to be typeof Function!');
            return this;
        }
        // Check if Callback Function already exists
        if (this.callbacks[key]) {
            internal_1.Agile.logger.error(`Event Callback Function with the key/name '${key}' already exists!`);
            return this;
        }
        this.callbacks[key] = _callback;
        return generateKey ? key : this;
    }
    //=========================================================================================================
    // Trigger
    //=========================================================================================================
    /**
     * @public
     * Triggers Events
     * @param payload - Payload that gets passed into the Callback Functions
     * @param keys - Keys of Callback Functions that get triggered (Note: if not passed all registered Events will be triggered)
     */
    trigger(payload, keys) {
        if (!this.enabled)
            return this;
        if (this.config.delay)
            this.delayedTrigger(payload, this.config.delay, keys);
        else
            this.normalTrigger(payload, keys);
        return this;
    }
    //=========================================================================================================
    // Disable
    //=========================================================================================================
    /**
     * @public
     * Disables Event
     */
    disable() {
        this.enabled = false;
        return this;
    }
    //=========================================================================================================
    // Enable
    //=========================================================================================================
    /**
     * @public
     * Enables Event
     */
    enable() {
        this.enabled = true;
        return this;
    }
    //=========================================================================================================
    // Reset
    //=========================================================================================================
    /**
     * @public
     * Resets Event
     */
    reset() {
        this.enabled = this.initialConfig.enabled;
        this.uses = 0;
        if (this.currentTimeout) {
            clearTimeout(this.currentTimeout);
            this.currentTimeout = undefined;
        }
        return this;
    }
    //=========================================================================================================
    // Remove Callback
    //=========================================================================================================
    /**
     * @public
     * Removes Callback Function at given Key
     * @param key - Key of Callback Function that gets removed
     */
    removeCallback(key) {
        delete this.callbacks[key];
        return this;
    }
    //=========================================================================================================
    // Normal Trigger
    //=========================================================================================================
    /**
     * @internal
     * Triggers normal Event
     * @param payload - Payload that gets passed into the Callback Functions
     * @param keys - Keys of Callback Functions that get triggered (Note: if not passed all registered Events will be triggered)
     */
    normalTrigger(payload, keys) {
        // Call wished Callback Functions
        if (!keys) {
            for (const key in this.callbacks)
                this.callbacks[key](payload);
        }
        else {
            for (const key of keys) {
                if (this.callbacks[key])
                    this.callbacks[key](payload);
            }
        }
        // Cause rerender
        if (this.config.rerender)
            this.observer.trigger();
        this.uses++;
        // Disable Event if maxUses got reached
        if (this.config.maxUses && this.uses >= this.config.maxUses)
            this.disable();
    }
    //=========================================================================================================
    // Delayed Trigger
    //=========================================================================================================
    /**
     * @internal
     * Triggers async Event (Events with a delay)
     * @param payload - Payload that gets passed into the Callback Functions
     * @param delay - Delay until Events get triggered
     * @param keys - Keys of Callback Functions that get triggered (Note: if not passed all registered Events will be triggered)
     */
    delayedTrigger(payload, delay, keys) {
        const eventJob = new internal_1.EventJob(payload, keys);
        // Execute Event no matter if another event is currently active
        if (this.config.overlap) {
            setTimeout(() => {
                this.normalTrigger(eventJob.payload, eventJob.keys);
            }, delay);
            return;
        }
        // Check if a Event(Timeout) is currently active if so add EventJob to queue
        if (this.currentTimeout !== undefined) {
            if (payload)
                this.queue.push(eventJob);
            return;
        }
        // Executes EventJob and calls itself again if queue isn't empty to execute the next EventJob
        const looper = (eventJob) => {
            this.currentTimeout = setTimeout(() => {
                this.currentTimeout = undefined;
                this.normalTrigger(eventJob.payload, eventJob.keys);
                if (this.queue.length > 0) {
                    const nextEventJob = this.queue.shift();
                    if (nextEventJob)
                        looper(nextEventJob);
                }
            }, delay);
        };
        looper(eventJob);
        return;
    }
}
exports.Event = Event;
