"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Integrations = void 0;
const internal_1 = require("../internal");
class Integrations {
    /**
     * @internal
     * Integrations - Manages Integrations of Agile
     * @param agileInstance - An Instance of Agile
     */
    constructor(agileInstance) {
        this.integrations = new Set(); // All registered Integrations
        this.agileInstance = () => agileInstance;
        // Integrate initial Integrations which are static and got set external
        internal_1.Agile.initialIntegrations.forEach((integration) => this.integrate(integration));
    }
    //=========================================================================================================
    // Integrate
    //=========================================================================================================
    /**
     * @internal
     * Integrates Framework(Integration) into Agile
     * @param integration - Integration/Framework that gets integrated
     */
    integrate(integration) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check if Integration is valid
            if (!integration._key) {
                internal_1.Agile.logger.error('Failed to integrate framework! Invalid Integration!', integration._key);
                return false;
            }
            // Bind Framework to Agile
            if (integration.methods.bind)
                integration.ready = yield integration.methods.bind(this.agileInstance());
            else
                integration.ready = true;
            // Integrate Framework
            this.integrations.add(integration);
            integration.integrated = true;
            // Logging
            internal_1.Agile.logger.info(`Successfully integrated '${integration._key}'`);
            return true;
        });
    }
    //=========================================================================================================
    // Update
    //=========================================================================================================
    /**
     * @internal
     * Updates registered and ready Integrations
     * -> calls 'updateMethod' in all registered and ready Integrations
     * @param componentInstance - Component that gets updated
     * @param updatedData - Properties that differ from the last Value
     */
    update(componentInstance, updatedData) {
        this.integrations.forEach((integration) => {
            if (!integration.ready) {
                internal_1.Agile.logger.warn(`Integration '${integration.key}' isn't ready yet!`);
                return;
            }
            if (integration.methods.updateMethod)
                integration.methods.updateMethod(componentInstance, updatedData);
        });
    }
    //=========================================================================================================
    // Has Integration
    //=========================================================================================================
    /**
     * @internal
     *  Check if at least one Integration got registered
     */
    hasIntegration() {
        return this.integrations.size > 0;
    }
}
exports.Integrations = Integrations;
