import { Agile, StorageKey, StateObserver, StatePersistent, Observer, PersistentKey, StateIngestConfigInterface, StateRuntimeJobConfigInterface } from '../internal';
export declare class State<ValueType = any> {
    agileInstance: () => Agile;
    _key?: StateKey;
    valueType?: string;
    isSet: boolean;
    isPlaceholder: boolean;
    initialStateValue: ValueType;
    _value: ValueType;
    previousStateValue: ValueType;
    nextStateValue: ValueType;
    observer: StateObserver<ValueType>;
    sideEffects: {
        [key: string]: (properties?: {
            [key: string]: any;
        }) => void;
    };
    computeMethod?: ComputeMethod<ValueType>;
    isPersisted: boolean;
    persistent: StatePersistent | undefined;
    watchers: {
        [key: string]: StateWatcherCallback<ValueType>;
    };
    /**
     * @public
     * State - Class that holds one Value and causes rerender on subscribed Components
     * @param agileInstance - An instance of Agile
     * @param initialValue - Initial Value of State
     * @param config - Config
     */
    constructor(agileInstance: Agile, initialValue: ValueType, config?: StateConfigInterface);
    /**
     * @public
     * Set Value of State
     */
    set value(value: ValueType);
    /**
     * @public
     * Get Value of State
     */
    get value(): ValueType;
    /**
     * @public
     * Set Key/Name of State
     */
    set key(value: StateKey | undefined);
    /**
     * @public
     * Get Key/Name of State
     */
    get key(): StateKey | undefined;
    /**
     * @internal
     * Updates Key/Name of State
     * @param value - New Key/Name of State
     */
    setKey(value: StateKey | undefined): this;
    /**
     * @public
     * Updates Value of State
     * @param value - new State Value
     * @param config - Config
     */
    set(value: ValueType, config?: StateRuntimeJobConfigInterface): this;
    /**
     * @internal
     * Ingests nextStateValue, computedValue into Runtime
     * @param config - Config
     */
    ingest(config?: StateIngestConfigInterface): this;
    /**
     * @public
     * Assign primitive type to State Value
     * Note: This function is mainly thought for JS users
     * @param type - wished Type ('String', 'Boolean', 'Array', 'Object', 'Number')
     */
    type(type: any): this;
    /**
     * @public
     * Undoes latest State Value change
     * @param config - Config
     */
    undo(config?: StateRuntimeJobConfigInterface): this;
    /**
     * @public
     * Resets State to its initial Value
     * @param config - Config
     */
    reset(config?: StateRuntimeJobConfigInterface): this;
    /**
     * @public
     * Patches Object with changes into State Value
     * Note: Only useful if State is an Object
     * @param targetWithChanges - Object that holds changes which get patched into State Value
     * @param config - Config
     */
    patch(targetWithChanges: object, config?: PatchConfigInterface): this;
    /**
     * @public
     * Watches State and detects State changes
     * @param callback - Callback Function that gets called if the State Value changes
     * @return Key of Watcher
     */
    watch(callback: StateWatcherCallback<ValueType>): string;
    /**
     * @public
     * Watches State and detects State changes
     * @param key - Key/Name of Watcher Function
     * @param callback - Callback Function that gets called if the State Value changes
     */
    watch(key: string, callback: StateWatcherCallback<ValueType>): this;
    /**
     * @public
     * Removes Watcher at given Key
     * @param key - Key of Watcher that gets removed
     */
    removeWatcher(key: string): this;
    /**
     * @public
     * Creates a Watcher that gets once called when the State Value changes for the first time and than destroys itself
     * @param callback - Callback Function that gets called if the State Value changes
     */
    onInaugurated(callback: StateWatcherCallback<ValueType>): void;
    /**
     * @public
     * Checks if watcher at given Key exists
     * @param key - Key/Name of Watcher
     */
    hasWatcher(key: string): boolean;
    /**
     * @public
     * Stores State Value into Agile Storage permanently
     * @param config - Config
     */
    persist(config?: StatePersistentConfigInterface): this;
    /**
     * @public
     * Stores State Value into Agile Storage permanently
     * @param key - Key/Name of created Persistent (Note: Key required if State has no set Key!)
     * @param config - Config
     */
    persist(key?: PersistentKey, config?: StatePersistentConfigInterface): this;
    /**
     * @public
     * Callback Function that gets called if the persisted Value gets loaded into the State for the first Time
     * Note: Only useful for persisted States!
     * @param callback - Callback Function
     */
    onLoad(callback: (success: boolean) => void): this;
    /**
     * @public
     * Creates fresh copy of State Value (-> No reference to State Value)
     */
    copy(): ValueType;
    /**
     * @public
     * Checks if State exists
     */
    get exists(): boolean;
    /**
     * @public
     * Equivalent to ===
     * @param value - Value that gets checked if its equals to the State Value
     */
    is(value: ValueType): boolean;
    /**
     * @public
     * Equivalent to !==
     * @param value - Value that gets checked if its not equals to the State Value
     */
    isNot(value: ValueType): boolean;
    /**
     * @public
     * Inverts State Value
     * Note: Only useful with boolean based States
     */
    invert(): this;
    /**
     * @public
     * Function that recomputes State Value if it changes
     * @param method - Computed Function
     */
    compute(method: ComputeMethod<ValueType>): this;
    /**
     * @internal
     * Adds SideEffect to State
     * @param key - Key/Name of SideEffect
     * @param sideEffect - Callback Function that gets called on every State Value change
     */
    addSideEffect(key: string, sideEffect: (properties?: {
        [key: string]: any;
    }) => void): this;
    /**
     * @internal
     * Removes SideEffect at given Key
     * @param key - Key of the SideEffect that gets removed
     */
    removeSideEffect(key: string): this;
    /**
     * @internal
     * Checks if sideEffect at given Key exists
     * @param key - Key of SideEffect
     */
    hasSideEffect(key: string): boolean;
    /**
     * @internal
     * Checks if Value has correct valueType (js)
     * Note: If no valueType set, it returns true
     * @param value - Value that gets checked for its correct Type
     */
    hasCorrectType(value: any): boolean;
    /**
     * @internal
     * Returns public Value of State
     */
    getPublicValue(): ValueType;
    /**
     * @internal
     * Returns Value that gets written into the Agile Storage
     */
    getPersistableValue(): any;
}
export declare type StateKey = string | number;
/**
 * @param key - Key/Name of State
 * @param deps - Initial deps of State
 * @param isPlaceholder - If State is initially a Placeholder
 */
export interface StateConfigInterface {
    key?: StateKey;
    deps?: Array<Observer>;
    isPlaceholder?: boolean;
}
/**
 * @param addNewProperties - If new Properties gets added to the State Value
 */
export interface PatchConfigInterface extends StateRuntimeJobConfigInterface {
    addNewProperties?: boolean;
}
/**
 * @param instantiate - If Persistent gets instantiated
 * @param storageKeys - Key/Name of Storages which gets used to persist the State Value (NOTE: If not passed the default Storage will be used)
 */
export interface StatePersistentConfigInterface {
    instantiate?: boolean;
    storageKeys?: StorageKey[];
}
export declare type StateWatcherCallback<T = any> = (value: T) => void;
export declare type ComputeMethod<T = any> = (value: T) => T;
