"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.State = void 0;
const internal_1 = require("../internal");
class State {
    /**
     * @public
     * State - Class that holds one Value and causes rerender on subscribed Components
     * @param agileInstance - An instance of Agile
     * @param initialValue - Initial Value of State
     * @param config - Config
     */
    constructor(agileInstance, initialValue, config = {}) {
        this.isSet = false; // If value is not the same as initialValue
        this.isPlaceholder = false;
        this.sideEffects = {}; // SideEffects of State (will be executed in Runtime)
        this.isPersisted = false; // If State can be stored in Agile Storage (-> successfully integrated persistent)
        this.watchers = {};
        config = internal_1.defineConfig(config, {
            deps: [],
            isPlaceholder: false,
        });
        this.agileInstance = () => agileInstance;
        this._key = config.key;
        this.observer = new internal_1.StateObserver(this, {
            key: config.key,
            deps: config.deps,
        });
        this.initialStateValue = internal_1.copy(initialValue);
        this._value = internal_1.copy(initialValue);
        this.previousStateValue = internal_1.copy(initialValue);
        this.nextStateValue = internal_1.copy(initialValue);
        this.isPlaceholder = true;
        // Initial Set
        if (!config.isPlaceholder)
            this.set(initialValue, { overwrite: true });
    }
    /**
     * @public
     * Set Value of State
     */
    set value(value) {
        this.set(value);
    }
    /**
     * @public
     * Get Value of State
     */
    get value() {
        internal_1.ComputedTracker.tracked(this.observer);
        return this._value;
    }
    /**
     * @public
     * Set Key/Name of State
     */
    set key(value) {
        this.setKey(value);
    }
    /**
     * @public
     * Get Key/Name of State
     */
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @internal
     * Updates Key/Name of State
     * @param value - New Key/Name of State
     */
    setKey(value) {
        var _a, _b;
        const oldKey = this._key;
        // Update State Key
        this._key = value;
        // Update Key in Observer
        this.observer._key = value;
        // Update Key in Persistent (only if oldKey equal to persistentKey -> otherwise the PersistentKey got formatted and will be set where other)
        if (value && ((_a = this.persistent) === null || _a === void 0 ? void 0 : _a._key) === oldKey)
            (_b = this.persistent) === null || _b === void 0 ? void 0 : _b.setKey(value);
        return this;
    }
    //=========================================================================================================
    // Set
    //=========================================================================================================
    /**
     * @public
     * Updates Value of State
     * @param value - new State Value
     * @param config - Config
     */
    set(value, config = {}) {
        config = internal_1.defineConfig(config, {
            sideEffects: true,
            background: false,
            force: false,
            storage: true,
            overwrite: false,
        });
        // Check value has correct Type (js)
        if (!this.hasCorrectType(value)) {
            const message = `Incorrect type (${typeof value}) was provided.`;
            if (!config.force) {
                internal_1.Agile.logger.error(message);
                return this;
            }
            internal_1.Agile.logger.warn(message);
        }
        // Ingest new value into Runtime
        this.observer.ingestValue(value, config);
        return this;
    }
    //=========================================================================================================
    // Ingest
    //=========================================================================================================
    /**
     * @internal
     * Ingests nextStateValue, computedValue into Runtime
     * @param config - Config
     */
    ingest(config = {}) {
        this.observer.ingest(config);
        return this;
    }
    //=========================================================================================================
    // Type
    //=========================================================================================================
    /**
     * @public
     * Assign primitive type to State Value
     * Note: This function is mainly thought for JS users
     * @param type - wished Type ('String', 'Boolean', 'Array', 'Object', 'Number')
     */
    type(type) {
        const supportedTypes = ['String', 'Boolean', 'Array', 'Object', 'Number'];
        // Check if type is a supported Type
        if (!supportedTypes.includes(type.name)) {
            internal_1.Agile.logger.warn(`'${type}' is not supported! Supported types: String, Boolean, Array, Object, Number`);
            return this;
        }
        this.valueType = type.name.toLowerCase();
        return this;
    }
    //=========================================================================================================
    // Undo
    //=========================================================================================================
    /**
     * @public
     * Undoes latest State Value change
     * @param config - Config
     */
    undo(config = {}) {
        this.set(this.previousStateValue, config);
        return this;
    }
    //=========================================================================================================
    // Reset
    //=========================================================================================================
    /**
     * @public
     * Resets State to its initial Value
     * @param config - Config
     */
    reset(config = {}) {
        this.set(this.initialStateValue, config);
        return this;
    }
    //=========================================================================================================
    // Patch
    //=========================================================================================================
    /**
     * @public
     * Patches Object with changes into State Value
     * Note: Only useful if State is an Object
     * @param targetWithChanges - Object that holds changes which get patched into State Value
     * @param config - Config
     */
    patch(targetWithChanges, config = {}) {
        config = internal_1.defineConfig(config, {
            addNewProperties: true,
            sideEffects: true,
            background: false,
            force: false,
            storage: true,
            overwrite: false,
        });
        if (!internal_1.isValidObject(this.nextStateValue)) {
            internal_1.Agile.logger.error("You can't use the patch method on a non object based States!");
            return this;
        }
        if (!internal_1.isValidObject(targetWithChanges)) {
            internal_1.Agile.logger.error('TargetWithChanges has to be an Object!');
            return this;
        }
        // Merge targetWithChanges into nextStateValue
        this.nextStateValue = internal_1.flatMerge(internal_1.copy(this.nextStateValue), targetWithChanges, { addNewProperties: config.addNewProperties });
        // Ingest updated nextStateValue into Runtime
        this.ingest({
            background: config.background,
            force: config.force,
            overwrite: config.overwrite,
            sideEffects: config.sideEffects,
            storage: config.storage,
        });
        return this;
    }
    watch(keyOrCallback, callback) {
        const generateKey = internal_1.isFunction(keyOrCallback);
        let _callback;
        let key;
        if (generateKey) {
            key = internal_1.generateId();
            _callback = keyOrCallback;
        }
        else {
            key = keyOrCallback;
            _callback = callback;
        }
        // Check if Callback is valid Function
        if (!internal_1.isFunction(_callback)) {
            internal_1.Agile.logger.error('A Watcher Callback Function has to be typeof Function!');
            return this;
        }
        // Check if watcherKey is already occupied
        if (this.watchers[key]) {
            internal_1.Agile.logger.error(`Watcher Callback Function with the key/name '${key}' already exists!`);
            return this;
        }
        this.watchers[key] = _callback;
        return generateKey ? key : this;
    }
    //=========================================================================================================
    // Remove Watcher
    //=========================================================================================================
    /**
     * @public
     * Removes Watcher at given Key
     * @param key - Key of Watcher that gets removed
     */
    removeWatcher(key) {
        delete this.watchers[key];
        return this;
    }
    /**
     * @public
     * Creates a Watcher that gets once called when the State Value changes for the first time and than destroys itself
     * @param callback - Callback Function that gets called if the State Value changes
     */
    onInaugurated(callback) {
        const watcherKey = 'InauguratedWatcherKey';
        this.watch(watcherKey, (value) => {
            callback(value);
            this.removeWatcher(watcherKey);
        });
    }
    //=========================================================================================================
    // Has Watcher
    //=========================================================================================================
    /**
     * @public
     * Checks if watcher at given Key exists
     * @param key - Key/Name of Watcher
     */
    hasWatcher(key) {
        return !!this.watchers[key];
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (internal_1.isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = this._key;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = internal_1.defineConfig(_config, {
            instantiate: true,
            storageKeys: [],
        });
        if (this.persistent)
            internal_1.Agile.logger.warn(`By persisting the State '${this._key}' twice you overwrite the old Persistent Instance!`);
        // Create persistent -> Persist Value
        this.persistent = new internal_1.StatePersistent(this, {
            instantiate: _config.instantiate,
            storageKeys: _config.storageKeys,
            key: key,
        });
        return this;
    }
    //=========================================================================================================
    // On Load
    //=========================================================================================================
    /**
     * @public
     * Callback Function that gets called if the persisted Value gets loaded into the State for the first Time
     * Note: Only useful for persisted States!
     * @param callback - Callback Function
     */
    onLoad(callback) {
        if (this.persistent) {
            this.persistent.onLoad = callback;
            // If State is already 'isPersisted' the loading was successful -> callback can be called
            if (this.isPersisted)
                callback(true);
        }
        else {
            internal_1.Agile.logger.error(`Please make sure you persist the State '${this._key}' before using the 'onLoad' function!`);
        }
        return this;
    }
    //=========================================================================================================
    // Copy
    //=========================================================================================================
    /**
     * @public
     * Creates fresh copy of State Value (-> No reference to State Value)
     */
    copy() {
        return internal_1.copy(this.value);
    }
    //=========================================================================================================
    // Exists
    //=========================================================================================================
    /**
     * @public
     * Checks if State exists
     */
    get exists() {
        return this._value !== undefined && !this.isPlaceholder;
    }
    //=========================================================================================================
    // Is
    //=========================================================================================================
    /**
     * @public
     * Equivalent to ===
     * @param value - Value that gets checked if its equals to the State Value
     */
    is(value) {
        return internal_1.equal(value, this.value);
    }
    //=========================================================================================================
    // Is Not
    //=========================================================================================================
    /**
     * @public
     * Equivalent to !==
     * @param value - Value that gets checked if its not equals to the State Value
     */
    isNot(value) {
        return internal_1.notEqual(value, this.value);
    }
    //=========================================================================================================
    // Invert
    //=========================================================================================================
    /**
     * @public
     * Inverts State Value
     * Note: Only useful with boolean based States
     */
    invert() {
        if (typeof this._value === 'boolean') {
            this.set(!this._value);
        }
        else {
            internal_1.Agile.logger.error('You can only invert boolean based States!');
        }
        return this;
    }
    //=========================================================================================================
    // Compute
    //=========================================================================================================
    /**
     * @public
     * Function that recomputes State Value if it changes
     * @param method - Computed Function
     */
    compute(method) {
        if (!internal_1.isFunction(method)) {
            internal_1.Agile.logger.error('A computeMethod has to be a function!');
            return this;
        }
        this.computeMethod = method;
        return this;
    }
    //=========================================================================================================
    // Add SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Adds SideEffect to State
     * @param key - Key/Name of SideEffect
     * @param sideEffect - Callback Function that gets called on every State Value change
     */
    addSideEffect(key, sideEffect) {
        if (!internal_1.isFunction(sideEffect)) {
            internal_1.Agile.logger.error('A sideEffect function has to be a function!');
            return this;
        }
        this.sideEffects[key] = sideEffect;
        return this;
    }
    //=========================================================================================================
    // Remove SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Removes SideEffect at given Key
     * @param key - Key of the SideEffect that gets removed
     */
    removeSideEffect(key) {
        delete this.sideEffects[key];
        return this;
    }
    //=========================================================================================================
    // Has SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Checks if sideEffect at given Key exists
     * @param key - Key of SideEffect
     */
    hasSideEffect(key) {
        return !!this.sideEffects[key];
    }
    //=========================================================================================================
    // Is Correct Type
    //=========================================================================================================
    /**
     * @internal
     * Checks if Value has correct valueType (js)
     * Note: If no valueType set, it returns true
     * @param value - Value that gets checked for its correct Type
     */
    hasCorrectType(value) {
        if (!this.valueType)
            return true;
        const type = typeof value;
        return type === this.valueType;
    }
    //=========================================================================================================
    // Get Public Value
    //=========================================================================================================
    /**
     * @internal
     * Returns public Value of State
     */
    getPublicValue() {
        // If State Value is used internal and output represents the real state value (for instance in Group)
        if (this['output'] !== undefined)
            return this['output'];
        return this._value;
    }
    //=========================================================================================================
    // Get Persistable Value
    //=========================================================================================================
    /**
     * @internal
     * Returns Value that gets written into the Agile Storage
     */
    getPersistableValue() {
        return this._value;
    }
}
exports.State = State;
