"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Persistent = void 0;
const internal_1 = require("../internal");
class Persistent {
    /**
     * @internal
     * Persistent - Handles storing of Agile Instances
     * Note: No stand alone class!!
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance, config = {}) {
        this.ready = false;
        this.isPersisted = false; // If Value is stored in Agile Storage
        // StorageKeys of Storages in that the Persisted Value gets saved
        this.storageKeys = [];
        this.agileInstance = () => agileInstance;
        this._key = Persistent.placeHolderKey;
        config = internal_1.defineConfig(config, {
            instantiate: true,
            storageKeys: [],
        });
        this.agileInstance().storages.persistentInstances.add(this);
        if (config.instantiate)
            this.instantiatePersistent({
                storageKeys: config.storageKeys,
                key: config.key,
            });
    }
    /**
     * @internal
     * Set Key/Name of Persistent
     */
    set key(value) {
        this.setKey(value);
    }
    /**
     * @internal
     * Get Key/Name of Persistent
     */
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @public
     * Sets Key/Name of Persistent
     * @param value - New Key/Name of Persistent
     */
    setKey(value) {
        this._key = value;
    }
    //=========================================================================================================
    // Instantiate Persistent
    //=========================================================================================================
    /**
     * @internal
     * Instantiates this Class
     * Note: Had to outsource it from the constructor because some extending classes
     * have to define some stuff before being able to instantiate the parent (this)
     * @param config - Config
     */
    instantiatePersistent(config = {}) {
        this._key = this.formatKey(config.key) || Persistent.placeHolderKey;
        this.assignStorageKeys(config.storageKeys);
        this.validatePersistent();
    }
    //=========================================================================================================
    // Validate Persistent
    //=========================================================================================================
    /**
     * @internal
     * Validates Persistent and updates its 'ready' property
     */
    validatePersistent() {
        let isValid = true;
        // Validate Key
        if (this._key === Persistent.placeHolderKey) {
            internal_1.Agile.logger.error('No valid persist Key found! Please provide a Key or assign one to the parent instance.');
            isValid = false;
        }
        // Validate StorageKeys
        if (!this.defaultStorageKey || this.storageKeys.length <= 0) {
            internal_1.Agile.logger.error('No persist Storage Key found! Please provide at least one Storage Key.');
            isValid = false;
        }
        this.ready = isValid;
        return isValid;
    }
    //=========================================================================================================
    // Assign StorageKeys
    //=========================================================================================================
    /**
     * @internal
     * Assign new StorageKeys to Persistent and overwrite the old ones
     * @param storageKeys - New Storage Keys
     */
    assignStorageKeys(storageKeys = []) {
        const storages = this.agileInstance().storages;
        // Set default Agile Storage to defaultStorage if no storageKey provided
        if (storageKeys.length <= 0) {
            this.storageKeys = [];
            if (storages.defaultStorage) {
                const key = storages.defaultStorage.key;
                this.defaultStorageKey = key;
                this.storageKeys.push(key);
            }
            return;
        }
        this.storageKeys = storageKeys;
        this.defaultStorageKey = storageKeys[0];
    }
    //=========================================================================================================
    // Initial Loading
    //=========================================================================================================
    /**
     * @internal
     * Loads/Saves Storage Value for the first Time
     */
    initialLoading() {
        return __awaiter(this, void 0, void 0, function* () {
            const success = yield this.loadPersistedValue();
            if (this.onLoad)
                this.onLoad(success);
            if (!success)
                yield this.persistValue();
        });
    }
    //=========================================================================================================
    // Load Value
    //=========================================================================================================
    /**
     * @internal
     * Loads Value from Storage
     * @return Success?
     */
    loadPersistedValue() {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.Agile.logger.error(`'loadPersistedValue' function isn't Set in Persistent! Be aware that Persistent is no stand alone class!`);
            return false;
        });
    }
    //=========================================================================================================
    // Update Value
    //=========================================================================================================
    /**
     * @internal
     * Saves/Updates Value in Storage
     * @return Success?
     */
    persistValue() {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.Agile.logger.error(`'persistValue' function isn't Set in Persistent! Be aware that Persistent is no stand alone class!`);
            return false;
        });
    }
    //=========================================================================================================
    // Remove Value
    //=========================================================================================================
    /**
     * @internal
     * Removes Value form Storage
     * @return Success?
     */
    removePersistedValue() {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.Agile.logger.error(`'removePersistedValue' function isn't Set in Persistent! Be aware that Persistent is no stand alone class!`);
            return false;
        });
    }
    //=========================================================================================================
    // Format Key
    //=========================================================================================================
    /**
     * @internal
     * Validates Storage Key
     * @param key - Key that gets validated
     */
    formatKey(key) {
        return key;
    }
}
exports.Persistent = Persistent;
Persistent.placeHolderKey = '__THIS_IS_A_PLACEHOLDER__';
