"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.globalBind = exports.clone = exports.generateId = exports.notEqual = exports.equal = exports.flatMerge = exports.defineConfig = exports.isJsonString = exports.isValidUrl = exports.isAsyncFunction = exports.isFunction = exports.getAgileInstance = exports.normalizeArray = exports.includesArray = exports.isValidObject = exports.copy = void 0;
const internal_1 = require("./internal");
//=========================================================================================================
// Copy
//=========================================================================================================
/**
 * @internal
 * Creates a fresh copy of an Array/Object
 * https://www.samanthaming.com/tidbits/70-3-ways-to-clone-objects/
 * @param value - Array/Object that gets copied
 */
function copy(value) {
    // Extra checking '!value' because 'typeof null === object'
    if (!value || typeof value !== 'object')
        return value;
    let temp;
    const newObject = Array.isArray(value) ? [] : {};
    for (const property in value) {
        temp = value[property];
        newObject[property] = typeof temp === 'object' ? copy(temp) : temp;
    }
    return newObject;
}
exports.copy = copy;
//=========================================================================================================
// Is Valid Object
//=========================================================================================================
/**
 * @internal
 * Checks if an value is a valid Object
 * https://stackoverflow.com/questions/12996871/why-does-typeof-array-with-objects-return-object-and-not-array
 * @param value - Value that is tested for its correctness
 */
function isValidObject(value) {
    function isHTMLElement(obj) {
        try {
            return obj instanceof HTMLElement;
        }
        catch (e) {
            return (typeof obj === 'object' &&
                obj.nodeType === 1 &&
                typeof obj.style === 'object' &&
                typeof obj.ownerDocument === 'object');
        }
    }
    return (value !== null &&
        typeof value === 'object' &&
        !isHTMLElement(value) &&
        !Array.isArray(value));
}
exports.isValidObject = isValidObject;
//=========================================================================================================
// Includes Array
//=========================================================================================================
/**
 * @internal
 * Check if array1 contains all elements of array2
 * @param array1 - Array 1
 * @param array2 - Array 2
 */
function includesArray(array1, array2) {
    return array2.every((element) => array1.includes(element));
}
exports.includesArray = includesArray;
//=========================================================================================================
// Normalize Array
//=========================================================================================================
/**
 * @internal
 * Transforms Item/s to an Item Array
 * @param items - Item/s that gets transformed to an Array
 * @param config - Config
 */
function normalizeArray(items, config = {}) {
    config = defineConfig(config, {
        createUndefinedArray: false,
    });
    if (!items && !config.createUndefinedArray)
        return [];
    return Array.isArray(items) ? items : [items];
}
exports.normalizeArray = normalizeArray;
//=========================================================================================================
// Get Instance
//=========================================================================================================
/**
 * @internal
 * Tries to get an Instance of Agile from provided Instance
 * If no agileInstance found it returns the global bound Agile Instance
 * @param instance - Instance that might hold an Agile Instance
 */
function getAgileInstance(instance) {
    try {
        // Try to get agileInstance from passed Instance
        if (instance) {
            const _agileInstance = isFunction(instance['agileInstance'])
                ? instance['agileInstance']()
                : instance['agileInstance'];
            if (_agileInstance)
                return _agileInstance;
        }
        // Return global bound agileInstance
        return globalThis['__agile__'];
    }
    catch (e) {
        internal_1.Agile.logger.error('Failed to get Agile Instance from ', instance);
    }
    return undefined;
}
exports.getAgileInstance = getAgileInstance;
//=========================================================================================================
// Is Function
//=========================================================================================================
/**
 * @internal
 * Checks if value is a function
 * @param value - Value that gets tested if its a function
 */
function isFunction(value) {
    return typeof value === 'function';
}
exports.isFunction = isFunction;
//=========================================================================================================
// Is Async Function
//=========================================================================================================
/**
 * @internal
 * Checks if value is an async function
 * @param value - Value that gets tested if its an async function
 */
function isAsyncFunction(value) {
    const valueString = value.toString();
    return (isFunction(value) &&
        (value.constructor.name === 'AsyncFunction' ||
            valueString.includes('__awaiter')));
}
exports.isAsyncFunction = isAsyncFunction;
//=========================================================================================================
// Is Valid Url
//=========================================================================================================
/**
 * @internal
 * Checks the correctness of an url
 * Resource: https://stackoverflow.com/questions/5717093/check-if-a-javascript-string-is-a-url
 * @param url - Url that gets tested for its correctness
 */
function isValidUrl(url) {
    const pattern = new RegExp('^(https?:\\/\\/)?' + // protocol
        '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' + // domain name
        '((\\d{1,3}\\.){3}\\d{1,3}))' + // OR ip (v4) address
        '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' + // port and path
        '(\\?[;&a-z\\d%_.~+=-]*)?' + // query string
        '(\\#[-a-z\\d_]*)?$', 'i');
    return pattern.test(url);
}
exports.isValidUrl = isValidUrl;
//=========================================================================================================
// Is Json String
//=========================================================================================================
/**
 * @internal
 * Checks if value is valid JsonString
 * @param value - Value that gets checked
 */
function isJsonString(value) {
    if (typeof value !== 'string')
        return false;
    try {
        JSON.parse(value);
    }
    catch (e) {
        return false;
    }
    return true;
}
exports.isJsonString = isJsonString;
//=========================================================================================================
// Define Config
//=========================================================================================================
/**
 * @internal
 * Merges default values/properties into config object
 * @param config - Config object that receives default values
 * @param defaults - Default values object that gets merged into config object
 * @param overwriteUndefinedProperties - If undefined Properties in config gets overwritten by the default value
 */
function defineConfig(config, defaults, overwriteUndefinedProperties) {
    if (overwriteUndefinedProperties === undefined)
        overwriteUndefinedProperties = true;
    if (overwriteUndefinedProperties) {
        const finalConfig = Object.assign(Object.assign({}, defaults), config);
        for (const key in finalConfig)
            if (finalConfig[key] === undefined)
                finalConfig[key] = defaults[key];
        return finalConfig;
    }
    return Object.assign(Object.assign({}, defaults), config);
}
exports.defineConfig = defineConfig;
/**
 * @internal
 * Merges items into object, be aware that the merge will only happen at the top level of the object
 * @param source - Source object
 * @param changes - Changes that get merged into the source object
 * @param config - Config
 */
function flatMerge(source, changes, config = {}) {
    // Copy Source to avoid references
    const _source = copy(source);
    if (!_source)
        return _source;
    // Loop through source object an merge changes into it
    const keys = Object.keys(changes);
    keys.forEach((property) => {
        if (!config.addNewProperties && !_source[property])
            return;
        _source[property] = changes[property];
    });
    return _source;
}
exports.flatMerge = flatMerge;
//=========================================================================================================
// Equals
//=========================================================================================================
/**
 * @internal
 * Check if two values are equal
 * @param value1 - First Value
 * @param value2 - Second Value
 */
function equal(value1, value2) {
    return value1 === value2 || JSON.stringify(value1) === JSON.stringify(value2);
}
exports.equal = equal;
//=========================================================================================================
// Not Equals
//=========================================================================================================
/**
 * @internal
 * Checks if two values aren't equal
 * @param value1 - First Value
 * @param value2 - Second Value
 */
function notEqual(value1, value2) {
    return !equal(value1, value2);
}
exports.notEqual = notEqual;
//=========================================================================================================
// Generate Id
//=========================================================================================================
/**
 * @internal
 * Generates random Id
 * @param length - Length of generated Id
 */
function generateId(length) {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    const charactersLength = characters.length;
    let result = '';
    if (!length)
        length = 5;
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
}
exports.generateId = generateId;
//=========================================================================================================
// Clone
//=========================================================================================================
/**
 * @internal
 * Clones a Class
 * @param instance - Instance of Class you want to clone
 */
function clone(instance) {
    // Clone Class
    const objectCopy = Object.create(Object.getPrototypeOf(instance));
    const objectClone = Object.assign(objectCopy, instance);
    // Copy Properties of Class
    for (const key in objectClone)
        objectClone[key] = copy(objectClone[key]);
    return objectClone;
}
exports.clone = clone;
//=========================================================================================================
// Global Bind
//=========================================================================================================
/**
 * @internal
 * Binds passed Instance globally at passed Key
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/globalThis
 * https://blog.logrocket.com/what-is-globalthis-why-use-it/
 * @param key - Key/Name of Instance
 * @param instance - Instance
 * @param overwrite - If already existing instance at passed Key gets overwritten
 */
function globalBind(key, instance, overwrite = false) {
    try {
        if (overwrite) {
            globalThis[key] = instance;
            return true;
        }
        if (!globalThis[key]) {
            globalThis[key] = instance;
            return true;
        }
    }
    catch (e) {
        internal_1.Agile.logger.error(`Failed to create global Instance called '${key}'`);
    }
    return false;
}
exports.globalBind = globalBind;
